#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"

#include <Trade\Trade.mqh>
CTrade              trade;

int handles[];
MqlTick previousTick, currentTick;


input string Symbols = "XAUUSD, GBPUSD, USDCAD, USDJPY";
input string Base_Quote = "USD";
input int Currecy_Main = 0;
string symb_List[];
string Formatted_Symbs[];
int Num_symbs = 0;
int takeProfit = 500;

//--- trailer
int trailingstopLoss = 200;
double StopLoss;

int OnInit(){

    string sprtr = ",";
    ushort usprtr;
    usprtr = StringGetCharacter(sprtr, 0);
    StringSplit(Symbols, usprtr, symb_List);
    Num_symbs = ArraySize(symb_List);
    ArrayResize(Formatted_Symbs, Num_symbs);
    
    for(int i = 0; i < Num_symbs; i++){
       Formatted_Symbs[i] = symb_List[i];
    }
    
    ArrayResize(handles, ArraySize(Formatted_Symbs));
    
    for(int i = 0; i < ArraySize(Formatted_Symbs); i++){
      handles[i] = iRSI(Formatted_Symbs[i], PERIOD_CURRENT, 14, PRICE_CLOSE);
      StopLoss = SymbolInfoDouble(Formatted_Symbs[i], SYMBOL_POINT)*trailingstopLoss;
    }
    return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason){
    
}

void OnTick(){
   
   if(isNewBar()){
      for(int i = 0; i < ArraySize(Formatted_Symbs); i++){
         Sig_trade(Formatted_Symbs[0], handles[0]);
         
         appliTS(Formatted_Symbs[i], StopLoss);
      }
   }
}

void appliTS(string symb, double StopLoss){
   for(int j = 0; j < ArraySize(Formatted_Symbs); j++){
      string thisSymb = Formatted_Symbs[j];
      
      static int digits = (int)SymbolInfoInteger(thisSymb, SYMBOL_DIGITS);
      
      // Trailing from the close prices
      double buysl = NormalizeDouble(SymbolInfoDouble(thisSymb, SYMBOL_BID)-StopLoss, digits);
      double sellsl = NormalizeDouble(SymbolInfoDouble(thisSymb, SYMBOL_ASK)+StopLoss, digits);
      
      int count = PositionsTotal();
      for(int i = count -1; i >= 0; i--){
         ulong ticket = PositionGetTicket(i);
         if(ticket > 0){
            if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY 
            && buysl > PositionGetDouble(POSITION_PRICE_OPEN) 
            && (PositionGetDouble(POSITION_SL)==0 || buysl>PositionGetDouble(POSITION_SL))){
               trade.PositionModify(ticket, buysl,0);
            } else
            if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL 
            && sellsl < PositionGetDouble(POSITION_PRICE_OPEN) 
            && (PositionGetDouble(POSITION_SL) == 0 || sellsl<PositionGetDouble(POSITION_SL))){
               trade.PositionModify(ticket, sellsl, 0);
            }
         }
      }
      
   }
}

void Sig_trade(string symb, int handler){
   
   double rsi[];
   CopyBuffer(handler, MAIN_LINE, 1, 1, rsi);
   
   // Dynamic position calculations
   double Lmaxs = SymbolInfoDouble(symb, SYMBOL_VOLUME_MAX);
   double Lmins = SymbolInfoDouble(symb, SYMBOL_VOLUME_MIN);
   double Lstep = SymbolInfoDouble(symb, SYMBOL_VOLUME_STEP);
   
   double volume = 0.0;
   double balance = AccountInfoDouble(ACCOUNT_EQUITY);
   
   volume = (0.01 / 100 * balance);
   
   // Ensure volume falls within permissible limits and conforms to the volume step
   volume = MathMax(Lmins, MathMin(Lmaxs, volume));
   volume = NormalizeDouble(volume / Lstep, 0) * Lstep;
   
   bool RSIBuy = rsi[0] < 30;
   bool RSISell = rsi[0] > 70;
   
   // Check if the current symbol is a base USD pair
   bool isBaseUSD = StringSubstr(symb, 0, 3) == "USD";
   bool isQuoteUSD = StringSubstr(symb, 3, 3) == "USD";
   string Bcurr = SymbolInfoString(symb, SYMBOL_CURRENCY_BASE);
   string Qcurr = SymbolInfoString(symb, SYMBOL_CURRENCY_PROFIT);
   
   double Bid = SymbolInfoDouble(symb, SYMBOL_BID);
   double Ask = SymbolInfoDouble(symb, SYMBOL_ASK);
   
   for(int i = PositionsTotal() - 1; i >= 0; i--){
      ulong posTicket = PositionGetTicket(i);
      if(PositionGetString(POSITION_SYMBOL) == symb){
         if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY){
            if(RSISell){
               trade.PositionClose(posTicket);
            }
            RSIBuy = false;
         }else if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL){
            if(RSIBuy){
               trade.PositionClose(posTicket);
            }
            RSISell = false;
         }
      }
   }
   
   
   
   for(int i = 0; i < ArraySize(Formatted_Symbs); i++){
      string currSymb = Formatted_Symbs[i];
      
      // Get base and quote currencies for the looped symbol
      string currBaseCurr = SymbolInfoString(currSymb, SYMBOL_CURRENCY_BASE);
      string currQuoteCurr = SymbolInfoString(currSymb, SYMBOL_CURRENCY_PROFIT);
      
      if(RSIBuy){
         
         if(currQuoteCurr == "USD"){
            trade.PositionOpen(currSymb, ORDER_TYPE_BUY, 0.02, currentTick.ask, 0, 0, "Correlation");
         }
         
         if(currBaseCurr == "USD"){
            trade.PositionOpen(currSymb, ORDER_TYPE_SELL, 0.02, currentTick.bid, 0, 0, "Correlation");
         }
      }
      
      if(RSISell){
         
         if(currBaseCurr == "USD"){
            trade.PositionOpen(currSymb, ORDER_TYPE_SELL, 0.02, currentTick.bid, 0, 0, "Correlation");
         }
         if(currQuoteCurr == "USD"){
            trade.PositionOpen(currSymb, ORDER_TYPE_BUY, 0.02, currentTick.ask, 0, 0, "Correlation");
         }
      }
   }
} 

bool isNewBar()
  {
//--- memorize the time of opening of the last bar in the static variable
   static datetime last_time=0;
//--- current time
   datetime lastbar_time= (datetime) SeriesInfoInteger(Symbol(),Period(),SERIES_LASTBAR_DATE); 

//--- if it is the first call of the function
   if(last_time==0)
     {
      //--- set the time and exit
      last_time=lastbar_time;
      return(false);
     }

//--- if the time differs
   if(last_time!=lastbar_time)
     {
      //--- memorize the time and return true
      last_time=lastbar_time;
      return(true);
     }
//--- if we passed to this line, then the bar is not new; return false
   return(false);
  }